(**
 * The First-Order LoopW Proof Checker base pretty-printer.
 *
 * The base pretty-printer converts terms, formulas, and ident abstract syntax into string.
 *
 * @author Tristan Crolard <crolard@univ-paris12.fr>
 * @author Emmanuel Polonowski <polonowski@univ-paris12.fr>
 * @version 0.2.4
 *)
signature BASE_PRETTY_PRINTER = sig

(**
 * builds a string with spaces for tabular display.
 *
 * @params n
 * @param n the number of tabs.
 *
 * @return the string tab.
 *)
val stringOfTab
	: int -> string

(**
 * converts terms abstract syntax into string.
 *
 * @params ott t
 * @param ott the ott flag.
 * @param t the term.
 *
 * @return the string representation of t.
 *)
val stringOfTerm
	: bool -> ProofAst.term -> string

(**
 * converts formulas/types abstract syntax into string accordingly to the formula/tex demand.
 *
 * @params form tex ott f
 * @param form the formula-disply (instead of imperative types) flag.
 * @param tex the tex flag.
 * @param ott the ott flag.
 * @param f the formula.
 *
 * @return the string representation of f.
 *)
val stringOfType
	: bool -> bool -> bool -> ProofAst.formula -> string

(**
 * converts quantified type list into string.
 *
 * @params form tex ott i il
 * @param form the formula-display (instead of imperative types) flag.
 * @param tex the tex flag.
 * @param ott the tex flag.
 * @param i the quantifier identifier list.
 * @param il the type list.
 *
 * @return the string representation of i il.
 *)
val stringOfQuantifiedTypeList
	: bool -> bool -> bool -> ProofAst.ident list -> ProofAst.formula list -> string

(**
 * converts identifiers into string.
 *
 * @params i
 * @param i the identifier.
 *
 * @return the string representation of i.
 *)
val stringOfIdent
	: ProofAst.ident -> string

(**
 * converts typed identifiers into string.
 *
 * @params form tex ott i
 * @param form the formula-disply (instead of imperative types) flag.
 * @param tex the tex flag.
 * @param ott the tex flag.
 * @param i the identifier.
 *
 * @return the string representation of i.
 *)
val stringOfTypedIdent
	: bool -> bool -> bool -> (ProofAst.ident * ProofAst.formula) -> string

(**
 * converts identifier lists into string.
 *
 * @params il
 * @param il the identifier list.
 *
 * @return the string representation of il.
 *)
val stringOfIdentList
	: ProofAst.ident list -> string

(**
 * converts typed identifier lists into string.
 *
 * @params form tex ott il
 * @param form the formula-display (instead of imperative types) flag.
 * @param tex the tex flag.
 * @param ott the tex flag.
 * @param il the typed identifier list.
 *
 * @return the string representation of il.
 *)
val stringOfTypedIdentList
	: bool -> bool -> bool -> (ProofAst.ident * ProofAst.formula) list -> string

(**
 * converts quantified typed identifier lists into string.
 *
 * @params form tex ott i il
 * @param form the formula-display (instead of imperative types) flag.
 * @param tex the tex flag.
 * @param ott the tex flag.
 * @param i the quantifier identifier list.
 * @param il the typed identifier list.
 *
 * @return the string representation of i il.
 *)
val stringOfQuantifiedTypedIdentList
	: bool -> bool -> bool -> ProofAst.ident list -> (ProofAst.ident * ProofAst.formula) list -> string

(**
 * converts parameter list into string accordingly to the formula/tex demand.
 *
 * @params form tex displayType idlist mode typidlist
 * @param form the formula-display (instead of imperative types) flag.
 * @param tex the tex flag.
 * @param displayType flag indicating whether types have to be displayed.
 * @param idlist the identifier/quantification list.
 * @param mode the mode in or out.
 * @param typidlist the typed identifier list of formal parameters.
 *
 * @return the string representation of the parameter list.
 *)
val stringOfParamList
	: bool -> bool -> bool -> ProofAst.ident list -> string
	  -> (ProofAst.ident * ProofAst.formula) list -> string

end

