(**
* The First-Order LoopW Proof Checker functional programs abstract syntax.
*
* The functionnal abstract syntax is used to represent annotated functional programs, i.e. CND proofs.
*
* @author Tristan Crolard <crolard@univ-paris12.fr>
* @author Emmanuel Polonowski <polonowski@univ-paris12.fr>
* @version 0.2.4
*)
structure FunAst = struct

(** identifiers as string (numeroted for alpha-conversion). *)
type ident = ProofAst.ident

(** first-order formulas. *)
datatype expType =
	(** absurd: _|_. *)
	  TBot
	(** nat predicate: nat(t) corresponds to "t in nat". *)
	| TNat of ProofAst.term
	(** equality predicate over terms: t = t'. *)
	| TEqual of ProofAst.term * ProofAst.term
	(** tuples: [i1, ..., in](f1, ..., fm) corresponds to "exists i1...in.(f1 & ... & fn)". *)
	| TTuple of ident list * expType list
	(** functions: [i1, ..., in](f1 -> f2) corresponds to "forall i1...in.(f1 => f2)". *)
	| TFun of ident list * expType * expType
	(** type variables *)
	| TVar of ident

(** functional terms. *)
datatype tterm =
	(** identifier. *)
	  Id of ident
	(** term tuples: [a1, ..., an](t1, ..., tm) (as existential introduction). *)
	| Tuple of ProofAst.term list * typed_term list
	(** zero. *)
	| Z
	(** successor constructor. *)
	| S of typed_term
	(** Succ function. *)
	| Succ of typed_term
	(** Pred function. *)
	| Pred of typed_term
	(** Application: (t[a1, ..., an] u) as universal instantiation. *)
	| App of typed_term * ProofAst.term list * typed_term
	(** Abstraction: fn (x1 : f1, ..., xp : fp) => t. *)
	| Abs of (ident * expType) list * typed_term
	(** Primitive recursion: rec(t1, t2, t3). *)
	| Rec of typed_term * typed_term * typed_term
	(** Let binding: let (x1 : f1 = u1, ..., xp : fp = up) in t (as existential elimination). *)
	| LetIn of (ident * expType) list * typed_term * typed_term
	(** Catch operator: call/cc (fn k => t). *)
	| Callcc of ident * typed_term
	(** Throw: throw t [a1, ..., an] u (with universal elimination). *)
	| Throw of typed_term * ProofAst.term list * typed_term
	(** Substitution: t {x/f[x]}[u : a = b] *)
	| Subst of typed_term * expType * ident * typed_term
	(** Lemma: f1, ..., fn |- f *)
	| Lemma of expType list * expType
	
withtype typed_term = tterm * expType

end

