(**
 * The First-Order LoopW Proof Checker pretty-printer keywords for abstract syntax.
 *
 * The pretty-printer keywords providing string representation of all keywords, accordingly to the tex flag.
 *
 * @author Tristan Crolard <crolard@univ-paris12.fr>
 * @author Emmanuel Polonowski <polonowski@univ-paris12.fr>
 * @version 0.2.4
 *)
structure PrettyPrinterKeywords :> PRETTY_PRINTER_KEYWORDS = struct

(** " value. *)
val doubleQuote = implode [#"\034"]
(** used to encapsulate tex-math string into "". *)
fun protect s = doubleQuote ^ s ^ doubleQuote
(** used to encapsulate tex-math string into $. *)
fun math s = protect ("$" ^ s ^ "$")

(**
 * forall keyword.
 *
 * @params tex
 * @param tex the tex flag.
 *
 * @return the string representation of forall.
 *)
fun kwdForall tex =
	if tex then math "\\forall" else "ForAll "

(**
 * exists keyword.
 *
 * @params tex
 * @param tex the tex flag.
 *
 * @return the string representation of exists.
 *)
fun kwdExists tex =
	if tex then math "\\exists" else "Exists "

(**
 * and keyword.
 *
 * @params tex
 * @param tex the tex flag.
 *
 * @return the string representation of and.
 *)
fun kwdAnd tex =
	if tex then math "\\wedge" else " & "

(**
 * implies keyword.
 *
 * @params tex
 * @param tex the tex flag.
 *
 * @return the string representation of implies.
 *)
fun kwdImplies tex =
	if tex then math "\\Rightarrow" else " => "

(**
 * neg keyword.
 *
 * @params tex
 * @param tex the tex flag.
 *
 * @return the string representation of neg.
 *)
fun kwdNeg tex =
	if tex then math "\\neg" else "~"

(**
 * top keyword.
 *
 * @params tex
 * @param tex the tex flag.
 *
 * @return the string representation of top.
 *)
fun kwdTop tex =
	if tex then math "\\top" else "T"

(**
 * bot keyword.
 *
 * @params tex
 * @param tex the tex flag.
 *
 * @return the string representation of bot.
 *)
fun kwdBot tex =
	if tex then math "\\bot" else "$"

(**
 * star keyword.
 *
 * @params tex
 * @param tex the tex flag.
 *
 * @return the string representation of star.
 *)
fun kwdStar tex =
	if tex then math "\\star" else "*"

(**
 * | keyword.
 *
 * @params tex
 * @param tex the tex flag.
 *
 * @return the string representation of |.
 *)
fun kwdVBar tex =
	if tex then math "\\mid" else "|"

(**
 * |- keyword.
 *
 * @params tex
 * @param tex the tex flag.
 *
 * @return the string representation of |-.
 *)
fun kwdVdash tex =
	if tex then math "\\vdash" else "|-"

(**
 * default keyword.
 *
 * @params tex
 * @param tex the tex flag.
 * @param k the keyword.
 *
 * @return the string representation of k.
 *)
fun kwd tex k =
	if tex then math ("\\mathbf{" ^ k ^ "}") else k

end

