(**
 * Utility function for the proof syntax.
 *
 * @author Tristan Crolard <crolard@univ-paris12.fr>
 * @author Emmanuel Polonowski <polonowski@univ-paris12.fr>
 * @version 0.2.4
 *)
structure ProofAstUtils :> PROOF_AST_UTILS = struct

open ProofAst

(** True type *)
val top = ProofAst.TEqual (ProofAst.FZero, ProofAst.FZero)

(**
 * builds the successor of a term.
 *
 * @params t
 * @param t the term.
 *
 * @return the term s(t).
 *)
fun s t = ProofAst.FFun (("s", 0), [t])

(**
 * builds the predecessor of a term.
 *
 * @params t
 * @param t the term.
 *
 * @return the term p(t).
 *)
fun p t = ProofAst.FFun (("p", 0), [t])

(**
 * builds an unary natural number from an int.
 *
 * @params n
 * @param n the integer value.
 *
 * @return the unary natural number s^n(Z).
 *)
fun mkNat n =
	if n = 0
	then ProofAst.FZero
	else ProofAst.FFun (("s", 0), [mkNat (n - 1)])

end

