(**
 * The First-Order LoopW Proof Checker proof syntax pretty-printer.
 *
 * The proof pretty-printer converts proof abstract syntax into string.
 *
 * @author Tristan Crolard <crolard@univ-paris12.fr>
 * @author Emmanuel Polonowski <polonowski@univ-paris12.fr>
 * @version 0.2.4
 *)
signature PROOF_PRETTY_PRINTER = sig

(**
 * converts program into string accordingly to the formula/tex demand.
 *
 * @params form tex p
 * @param form the formula-disply (instead of imperative types) flag.
 * @param tex the tex flag.
 * @param p the program.
 *
 * @return the string representation of p.
 *)
val stringOfProgram
	: bool -> bool -> ProofAst.program -> string list

(**
 * converts program into string, generating ott-oriented type and syntax.
 *
 * @params p
 * @param p the program.
 *
 * @return the string representation of p.
 *)
val stringOfProgram_Ott
	: ProofAst.program -> string list

(**
 * converts program into string accordingly to the formula/tex demand,
 * without any type information.
 *
 * @params form tex p
 * @param form the formula-disply (instead of imperative types) flag.
 * @param tex the tex flag.
 * @param p the program.
 *
 * @return the string representation of p.
 *)
val stringOfProgram_Untyped
	: bool -> bool -> ProofAst.program -> string list

(**
 * converts expression into string.
 *
 * @params e
 * @param e the expression.
 *
 * @return the string representation of e.
 *)
val stringOfExpression
	: ProofAst.expression -> string

end

