(**
 * The First-Order LoopW Type Inference compiler.
 *
 * The compiler calls several modules in order to
 * perform the requested tasks.
 *
 * @author Tristan Crolard <crolard@univ-paris12.fr>
 * @author Emmanuel Polonowski <polonowski@univ-paris12.fr>
 * @version 0.2.4
 *)
signature COMPILER = sig

(**
 * The type of compiling parameters.
 *)
type compilerParams = 
	{ (** the path of the source file which contains the imperative program ".loop". *)
	  sourceFile : string,
	  (** the source file name prefix. *)
	  fileNamePrefix : string,
	  (** the verbosity level (0: nothing, ..., 3: very verbose). *)
	  verbLevel : int,
	  (** asks the compiler to write the untyped program in a file ".cs". *)
	  printUntyped : bool,
	  (** asks the compiler to write the typed program in a file ".rev.loop". *)
	  printTyped : bool,
	  (** asks the compiler to write the complete infered proof in a file ".proof". *)
	  printProof : bool,
	  (** asks the compiler to write the types as formulas. *)
	  formula : bool,
	  (** asks the compiler to write the output for further tex compilation. *)
	  tex : bool,
	  (** asks the compiler to write the complete infered proof in a file ".loop.ott" for ott/twelf compilation. *)
	  ott : bool
	}

(**
 * performs type inference and on-demand functional translation.
 *
 * @params params
 * @param params the compiling parameters.
 *)
val compile
	: compilerParams -> unit

end

