(**
 * Utility function for the core syntax.
 *
 * @author Tristan Crolard <crolard@univ-paris12.fr>
 * @author Emmanuel Polonowski <polonowski@univ-paris12.fr>
 * @version 0.2.4
 *)
signature CORE_AST_UTILS = sig

(** True type *)
val top
	: ProofAst.formula

(**
 * builds the successor of a term.
 *
 * @params t
 * @param t the term.
 *
 * @return the term s(t).
 *)
val s
	: ProofAst.term -> ProofAst.term

(**
 * builds the predecessor of a term.
 *
 * @params t
 * @param t the term.
 *
 * @return the term p(t).
 *)
val p
	: ProofAst.term -> ProofAst.term

(**
 * builds an unary natural number from an int.
 *
 * @params n
 * @param n the integer value.
 *
 * @return the unary natural number s^n(Z).
 *)
val mkNat
	: int -> ProofAst.term

(**
 * removes regions in expressions abstract syntax.
 *
 * @params e
 * @param e the expression.
 *
 * @return the expression without any region.
 *)
val removeRegionExpression
	: CoreAst.expression -> CoreAst.expression

(**
 * removes regions in programs abstract syntax.
 *
 * @params p
 * @param p the program.
 *
 * @return the program without any region.
 *)
val removeRegionProgram
	: CoreAst.program -> CoreAst.program

end

