(**
 * The First-Order LoopW Type Inference core syntax pretty-printer.
 *
 * The core pretty-printer converts core abstract syntax into string.
 *
 * @author Tristan Crolard <crolard@univ-paris12.fr>
 * @author Emmanuel Polonowski <polonowski@univ-paris12.fr>
 * @version 0.2.4
 *)
signature CORE_PRETTY_PRINTER = sig

(**
 * converts value into string accordingly to the formula/tex demand.
 *
 * @params form tex v
 * @param form the formula-disply (instead of imperative types) flag.
 * @param tex the tex flag.
 * @param v the value.
 *
 * @return the string representation of v.
 *)
val stringOfValue
	: bool -> bool -> CoreAst.value -> string list

(**
 * converts expression into string accordingly to the formula/tex demand.
 *
 * @params form tex e
 * @param form the formula-disply (instead of imperative types) flag.
 * @param tex the tex flag.
 * @param e the expression.
 *
 * @return the string representation of e.
 *)
val stringOfExpression
	: bool -> bool -> CoreAst.expression -> string list

(**
 * converts command into string accordingly to the formula/tex demand.
 *
 * @params form tex c
 * @param form the formula-disply (instead of imperative types) flag.
 * @param tex the tex flag.
 * @param c the command.
 *
 * @return the string representation of c.
 *)
val stringOfCommand
	: bool -> bool -> CoreAst.command -> string list

(**
 * converts sequence into string accordingly to the formula/tex demand.
 *
 * @params form tex s
 * @param form the formula-disply (instead of imperative types) flag.
 * @param tex the tex flag.
 * @param s the sequence.
 *
 * @return the string representation of s.
 *)
val stringOfSequence
	: bool -> bool -> CoreAst.sequence -> string list

(**
 * converts program into string accordingly to the formula/tex demand.
 *
 * @params form tex p
 * @param form the formula-disply (instead of imperative types) flag.
 * @param tex the tex flag.
 * @param p the program.
 *
 * @return the string representation of p.
 *)
val stringOfProgram
	: bool -> bool -> CoreAst.program -> string list

(**
 * converts program into string accordingly to the formula/tex demand,
 * without any type information.
 *
 * @params form tex p
 * @param form the formula-disply (instead of imperative types) flag.
 * @param tex the tex flag.
 * @param p the program.
 *
 * @return the string representation of p.
 *)
val stringOfProgram_Untyped
	: bool -> bool -> CoreAst.program -> string list

end
