(**
 * The First-Order LoopW Type Inference syntax error module.
 *
 * The error module exports the region type and function to display errors.
 *
 * @author Tristan Crolard <crolard@univ-paris12.fr>
 * @author Emmanuel Polonowski <polonowski@univ-paris12.fr>
 * @version 0.2.4
 *)
structure Error = struct

(**
 * store information about the file and the position in the file.
 *)
type region = string * (int * int) * (int * int)

(**
 * converts line and column into a string respresentation.
 *
 * @params (l,c)
 * @param (l,c) the pair line-column.
 *
 * @return the string representation "[l].[c]".
 *)
fun toString (l, c) = (Int.toString l) ^ "." ^ (Int.toString c)

(**
 * converts a region into a string.
 *
 * @params reg
 * @param reg the region.
 *
 * @return the string representation "fileName:[l1].[c1]-[l2].[c2]".
 *)
fun display (name, p1, p2) =
	name ^ ":" ^ (toString p1) ^ "-" ^ (toString p2)

(**
 * displays the error message and region, and raise a failure.
 *
 * @params reg msg
 * @param reg the region.
 * @param msg the error message.
 *)
fun fail (reg: region) (msg: string) = let
in
	print ((display reg) ^ " Error: " ^ msg ^ "\n");
	raise Fail "aborting."
end

(**
 * displays the error message, and raise a failure.
 *
 * @params msg
 * @param msg the error message.
 *)
fun failm (msg: string) = let
in
	print ("Error: " ^ msg ^ "\n");
	raise Fail "aborting."
end

(**
 * calls either fail or failm depending on the region Option.
 *
 * @params regOpt msg
 * @param regOpt the region option.
 * @param msg the error message.
 *)
fun optFail (regOpt: region option) (msg: string) =
	case regOpt of
		NONE => failm msg
      | SOME r => fail r msg
end

