(*
 * Glue code for putting together the output of ml-lex and ml-yacc.
 *)
structure Parser = struct

  structure SyntaxLrVals
    = SyntaxLrValsFun (structure Token = LrParser.Token)

  structure SyntaxLexer
    = SyntaxLexFun (structure Tokens = SyntaxLrVals.Tokens)

  structure SyntaxParser
    = JoinWithArg (structure ParserData = SyntaxLrVals.ParserData
  (**)             structure Lex = SyntaxLexer
  (**)             structure LrParser = LrParser)

  open SyntaxLexer.UserDeclarations

  structure T = SyntaxLrVals.Tokens
  structure EC = SyntaxLrVals.ParserData.EC
  structure Token = SyntaxLrVals.ParserData.Token
  structure Actions = SyntaxLrVals.ParserData.Actions

  fun termToKeyword t (p: pos, q: pos) = Token.TOKEN (t, (Actions.void, p, q))

  fun termToString t = String.map Char.toLower (EC.showTerminal t)

  val keywords : (string * (pos * pos -> lexresult)) list =
      map (fn t => (termToString t, termToKeyword t))
	  (List.filter EC.is_keyword EC.terms)

  fun symb n = termToKeyword (List.nth (EC.terms, n))

  infix ==
  fun (t1 == t2) = SyntaxParser.sameToken (t1(0, 0), t2(0, 0))

  val (lineComment, blockComment) = let
      val (s0, s1, s2, s3) = (symb 0, symb 1, symb 2, symb 3)
  in
      if s3 == T.EOF then ((fn t => t == s0), (fn t => t == s1))
      else if s2 == T.EOF then ((fn _ => false), (fn t => t == s0))
      else if s1 == T.EOF then ((fn t => t == s0), (fn _ => false))
      else ((fn _ => false), (fn _ => false))
  end

  fun lexerror (pos1, pos2) msg =
      Error.fail (locate (pos1, pos2)) ("lexer failure: " ^ msg)

  fun error (msg, pos1, pos2) =
      Error.fail (locate (pos1, pos2)) msg

  fun parseStream bc lc stream = let
      fun get _ = TextIO.input stream
      val lexer = SyntaxParser.makeLexer get (bc, lc, keywords, lexerror)
      val (ast, _) = SyntaxParser.parse (30, lexer, error, locate)
  in ast end

  fun parseString source = let
      val () = initLexer "stdIn"
      val stream = TextIO.openString source
      val ast = parseStream blockComment lineComment stream
      val () = TextIO.closeIn stream
  in ast end

  fun parseFile filename = let
      val () = initLexer filename
      val stream = TextIO.openIn filename
      val ast = parseStream blockComment lineComment stream
      val () = TextIO.closeIn stream
  in ast end

end
