(**
 * Utility functions that deals with lists.
 *
 * @author Tristan Crolard <crolard@univ-paris12.fr>
 * @author Emmanuel Polonowski <polonowski@univ-paris12.fr>
 * @version 0.2.4
 *)
signature UTILS = sig

(**
 * check if the given element belongs to the list.
 *
 * @params e l
 * @param e the element.
 * @param l the list.
 *
 * @return true if e belongs to l, else false.
 *)
val listMember
	: ''a -> ''a list -> bool

(**
 * merges the two list without duplicating identical elements.
 *
 * @params l1 l2
 * @param l1 the first list.
 * @param l2 the second list.
 *
 * @return l1 followed by (l2 - l1).
 *)
val listFusion
	: ''a list -> ''a list -> ''a list

(**
 * merges the two list without duplicating identical elements.
 *
 * @params l1 l2
 * @param l1 the first list.
 * @param l2 the second list.
 *
 * @return l1 followed by (l2 - l1).
 *)
val listFusionPair
	: (''a list * ''a list) -> ''a list

(**
 * check if the given element belongs to the list, and removes it.
 *
 * @params e l
 * @param e the element.
 * @param l the list.
 *
 * @return ( (true if e belongs to l, else false) , (l without e) ).
 *)
val listMemberRemove
	: ''a -> ''a list -> (bool * ''a list)

(**
 * check if the two lists contains the same elements (not considering their
 * position in the list).
 *
 * @params l1 l2
 * @param l1 the first list.
 * @param l2 the second list.
 *
 * @return true if and only if l1 and l2 contains the same elements.
 *)
val listEquals
	: ''a list -> ''a list -> bool

(**
 * flattens a list of list, inserting a separator between each inner lists.
 *
 * @params sep l
 * @param sep the separator (itself a list).
 * @param l the list list [l1, l2, ..., ln].
 *
 * @return l1 @ sep @ l2 @ ... @ ln.
 *)
val listConcatWith
	: 'a list -> 'a list list -> 'a list

end

